import os,sys
import traceback

import xml.etree.ElementTree as xmlet
import numpy as np
import csv

from PyQt5.QtCore import pyqtSignal,QPointF,Qt,QCoreApplication
from PyQt5 import QtGui
import PyQt5.QtWidgets as qt
from PyQt5 import QtCore

import UsefullVariables as vrb
import UsefullWidgets as wgt
import UsefullFunctions as fct
import DatabaseFunction as Dfct

import PyIPSDK
import PyIPSDK.IPSDKIPLMorphology as morpho
import PyIPSDK.IPSDKIPLAdvancedMorphology as advmorpho
import PyIPSDK.IPSDKIPLColor as color
import PyIPSDK.IPSDKIPLUtility as util
import PyIPSDK.IPSDKIPLFiltering as filtering
import PyIPSDK.IPSDKIPLBinarization as bin
import PyIPSDK.IPSDKIPLArithmetic as arithm
import PyIPSDK.IPSDKIPLGlobalMeasure as glbmsr
import PyIPSDK.IPSDKIPLGeometricTransform as gtrans
import PyIPSDK.IPSDKIPLShapeAnalysis as shapeanalysis
import PyIPSDK.IPSDKIPLShapeSegmentation as shapesegmentation
import PyIPSDK.IPSDKIPLIntensityTransform as itrans
import PyIPSDK.IPSDKIPLLogical as logic
import PyIPSDK.IPSDKIPLFeatureDetection as fd
import PyIPSDK.IPSDKIPLStats as stats
import PyIPSDK.IPSDKIPLClassification as classif
import PyIPSDK.IPSDKIPLRegistration as registration
import PyIPSDK.IPSDKIPLMachineLearning as ml

import PyIPSDK.IPSDKUI as ui

mainWindow = vrb.mainWindow

try:
    groupMenu = mainWindow.groupMenu

    buttonMacro = wgt.PushButtonImage(vrb.folderMacroInterface + "/Macro_BatchSmartSegmentation/BatchSmartSegmentation_Image.png")
    buttonMacro.setFixedSize(30*vrb.ratio, 30*vrb.ratio)

    groupMenu.layoutBar1.addWidget(buttonMacro, 0, vrb.numMacro, Qt.AlignVCenter)
    vrb.numMacro += 1
except:
    pass

class FileSelector(qt.QGroupBox):

    def __init__(self, label):
        qt.QGroupBox.__init__(self)

        self.labelFile = qt.QLabel(label)
        self.lineEditFile = qt.QLineEdit()
        self.buttonOpen = wgt.PushButtonImage(vrb.folderImages + "/Folder.png")
        self.buttonOpen.setFixedSize(20*vrb.ratio, 20*vrb.ratio)

        self.layout = qt.QGridLayout()
        self.layout.addWidget(self.labelFile,0,0)
        self.layout.addWidget(self.lineEditFile,0,1)
        self.layout.addWidget(self.buttonOpen,0,2)
        self.setLayout(self.layout)

        self.layout.setContentsMargins(0,0,0,0)

        style = fct.getStyleSheet()
        self.setStyleSheet(style)

class SettingsGroupBox(qt.QGroupBox):

    def __init__(self):
        qt.QGroupBox.__init__(self)

        self.labelModel = qt.QLabel("Model")
        self.labelModel.setFixedWidth(55*vrb.ratio)
        self.comboboxModel = qt.QComboBox()
        self.comboboxModel.setFixedWidth(120*vrb.ratio)
        self.createModelList()

        self.checkBoxColorOutput = qt.QCheckBox("Color output image")
        self.checkBoxColorOutput.setVisible(True)

        self.layout = qt.QGridLayout()
        self.layout.addWidget(self.labelModel,0,0)
        self.layout.addWidget(self.comboboxModel,0,1,Qt.AlignLeft)
        self.layout.addWidget(self.checkBoxColorOutput,1,0, 2, 1, Qt.AlignLeft)
        self.setLayout(self.layout)

        self.layout.setContentsMargins(10,10,10,10)

#        self.setTitle("Réglages")
#        self.setStyleSheet('QGroupBox:title {left: 20px ;padding-left: 10px;padding-right: 10px; padding-top: -12px; color:rgb(6, 115, 186)}  QGroupBox {font: bold; border: 1px solid gray; margin-top: 12 px}')

    def createModelList(self):
        folder = os.path.dirname(vrb.folderMacroInterface) + "/Machine_Learning/Pixel_Classification"

        self.comboboxModel.clear()
        for dirElt in os.listdir(folder):
            if os.path.isdir(os.path.join(folder, dirElt)):
                self.comboboxModel.addItem(dirElt)


class InformationWidget(qt.QWidget):

    def __init__(self):
        qt.QWidget.__init__(self)

        try:
            self.setWindowFlag(Qt.WindowStaysOnTopHint)
        except:
            self.setWindowFlags(Qt.WindowStaysOnTopHint)

        try:
            file = xmlet.parse(vrb.folderInformation + "/userMacro.mho")
            self.settingsElement = file.getroot()
        except:
            self.settingsElement = xmlet.Element('userMacro')

        self.dictResult = {}

        self.fileSelector = FileSelector("Input directory")
        self.fileSelector.setFixedHeight(30*vrb.ratio)
        self.settingsGroupBox = SettingsGroupBox()
        self.buttonValidate = wgt.PushButtonImage(vrb.folderImages + "/Validate.png")
        self.buttonValidate.setFixedSize(30*vrb.ratio,30*vrb.ratio)

        self.layout = qt.QGridLayout()
        self.layout.addWidget(self.fileSelector, 0, 0,1,2)
        self.layout.addWidget(self.settingsGroupBox, 1, 0,1,2)
        self.layout.addWidget(self.buttonValidate, 2, 1)

        self.setLayout(self.layout)

        style = fct.getStyleSheet()
        self.setStyleSheet(style)

        self.fileSelector.buttonOpen.clicked.connect(self.openQDialog)
        self.buttonValidate.clicked.connect(self.process)

        self.setWindowTitle("Settings")

        self.setFixedSize(250*vrb.ratio,150*vrb.ratio)

    def openQDialog(self):

        defaultFolder = Dfct.childText(vrb.userPathElement, "CustomImport")
        if defaultFolder is None or defaultFolder == "" or os.path.exists(defaultFolder) == False:
            defaultFolder = "C:/"
        filename = qt.QFileDialog.getExistingDirectory(self, "Select your folder", defaultFolder)
        if filename != "" and filename != None:
            try:
                self.fileSelector.lineEditFile.setText(filename)
                Dfct.SubElement(vrb.userPathElement, "CustomImport").text = os.path.dirname(filename)
                Dfct.saveXmlElement(vrb.userPathElement, vrb.folderInformation + "/UserPath.mho", forceSave=True)
            except:
                pass

    def process(self):

        # import time
        # start = time.time()

        try:
            vrb.mainWindow.toggleGroupBoxProcessing()
            vrb.mainWindow.groupBoxProcessing.setText('')
            vrb.mainWindow.groupBoxProcessing.buttonStop.setVisible(False)
            vrb.mainWindow.groupBoxProcessing.progressBar.setVisible(False)
        except:
            pass

        self.dictResult = {}

        try:
            # Retrieve the input directory
            folder = self.fileSelector.lineEditFile.text()

            # Load the model
            modelName = self.settingsGroupBox.comboboxModel.currentText()
            modelPath = PyIPSDK.getUserRootDataDir() + "/ReactivIP/Explorer/Machine_Learning/Pixel_Classification/" + modelName + "/ModelIPSDK.bin"
            model = PyIPSDK.readRandomForestModel(modelPath)

            # Check if the directory exists
            if folder != "" and os.path.exists(folder):
                # Create the output sub-directory
                outputFolder = self.fileSelector.lineEditFile.text() + "/Output"
                if not os.path.exists(outputFolder):
                    os.makedirs(outputFolder)

                # For all tiff images in folder
                cpt = 0
                for imgName in os.listdir(folder):
                    if imgName.endswith(".tif") or imgName.endswith(".tiff") or imgName.endswith(".TIF") or imgName.endswith(".TIFF"):
                        # Load the image
                        img = PyIPSDK.loadTiffImageFile(os.path.join(folder, imgName))

                        # Process the image
                        outImg = ml.pixelClassificationRFImg(img,model)

                        # Save the result
                        PyIPSDK.saveTiffImageFile(os.path.join(outputFolder, "out_"+imgName), outImg)

                        # Also save the result as a color image?
                        if self.settingsGroupBox.checkBoxColorOutput.isChecked():
                            outColorImg = self.labelToColor(outImg)
                            PyIPSDK.saveTiffImageFile(os.path.join(outputFolder, "outColor_"+imgName), outColorImg)
                        cpt+=1

                messageBox = wgt.MessageBox(str(cpt) + " images has been processed\nThe results are stored in the directory:\n"+outputFolder, '', buttons=[qt.QMessageBox.Ok], icon=qt.QMessageBox.Information, windowTitle="Information")
                messageBox.exec()
        except:

            messageBox = wgt.MessageBox("An error occurred during the process.\nPlease, check the settings.", '', buttons=[qt.QMessageBox.Ok], icon=qt.QMessageBox.Warning, windowTitle="Error")
            messageBox.exec()

            traceback.print_exc(file=sys.stderr)

        try:
            vrb.mainWindow.toggleGroupBoxProcessing()
        except:
            pass

    def labelToColor(self, im):

        # Create the ouput image geometry
        geometry = None
        bIs2d = im.getVolumeGeometryType() == PyIPSDK.eVGT_2d
        if bIs2d:
            geometry = PyIPSDK.geometryRgb2d(im.getBufferType(), im.getSizeX(), im.getSizeY())
        else:
            geometry = PyIPSDK.geometryRgb3d(im.getBufferType(), im.getSizeX(), im.getSizeY(), im.getSizeZ())

        # Create the output color image
        imColor = None
        if im.getColorGeometryType() == PyIPSDK.eCGT_Grey:
            imColor = PyIPSDK.createImage(geometry)

            if bIs2d:
                # Copy the channels
                for c in range(0, 3):
                    plan = PyIPSDK.extractPlan(0, c, 0, imColor)
                    util.copyImg(im, plan)
            else:
                # Copy the channels
                for c in range(0, 3):
                    plan = PyIPSDK.extractVolume(c, 0, imColor)
                    util.copyImg(im, plan)
        else:
            imColor = util.copyImg(im)

        # Count the number of labels
        statsRes = None
        if bIs2d:
            statsRes = glbmsr.statsMsr2d(im)
        else:
            statsRes = glbmsr.statsMsr3d(im)
        nbLabels = statsRes.max

        # For label images, we create 3 random LUTs (one per channel)
        # 2 with values between 50 and 200
        channelValues = np.random.rand(3, int(nbLabels + 1)) * 150 + 50

        lutR = PyIPSDK.createIntensityLUT(0, 1, channelValues[0, :])
        lutG = PyIPSDK.createIntensityLUT(0, 1, channelValues[1, :])
        lutB = PyIPSDK.createIntensityLUT(0, 1, channelValues[2, :])

        # The background is set to 0
        lutR.lookupTable[0] = 0
        lutG.lookupTable[0] = 0
        lutB.lookupTable[0] = 0

        # Special case for binary image : we only  use a blue color for value 1
        if im.getBufferType() == PyIPSDK.eIBT_Binary:
            lutR.lookupTable[1] = 0
            lutG.lookupTable[1] = 0
            lutB.lookupTable[1] = 200

        colorLut = [lutR, lutG, lutB]

        # Convert the label image to a color image
        if bIs2d:
            for c in range(0, 3):
                plan = PyIPSDK.extractPlan(0, c, 0, imColor)
                itrans.lutTransform2dImg(im, colorLut[c], plan)
        else:
            for c in range(0, 3):
                plan = PyIPSDK.extractVolume(c, 0, imColor)
                itrans.lutTransform3dImg(im, colorLut[c], plan)

        imColor = util.convertImg(imColor, PyIPSDK.eIBT_UInt8)
        return imColor

informationWidget = InformationWidget()

def openWidget():
    informationWidget.settingsGroupBox.createModelList()
    fct.showWidget(informationWidget)

try:
    buttonMacro.clicked.connect(openWidget)
except:
    pass

if __name__ == '__main__':
    import sys
    app = QtCore.QCoreApplication.instance()
    if app is None:
        app = qt.QApplication([])

    sys._excepthook = sys.excepthook


    def exception_hook(exctype, value, traceback):
        print(exctype, value, traceback)
        sys._excepthook(exctype, value, traceback)
        sys.exit(1)


    sys.excepthook = exception_hook

    foo = InformationWidget()
    # foo.resize(400,600)
    foo.show()

    app.exec_()